<?php

function icl_node_filter_form() {
  module_load_include('inc', 'node', 'node.admin');
  
  $session = &$_SESSION['icl_dashboard_overview_filter'];
  $session = is_array($session) ? $session : array();  
  $filters = node_filters();

  $i = 0;
  $form['filters'] = array(
    '#type' => 'fieldset',
    '#title' => t('Show only items where'),
    '#theme' => 'node_filters',
  );
  $form['#submit'][] = 'icl_dashboard_filter_form_submit';
  foreach ($session as $filter) {
    list($type, $value) = $filter;
    if ($type == 'category') {
      // Load term name from DB rather than search and parse options array.
      $value = module_invoke('taxonomy', 'get_term', $value);
      $value = $value->name;
    }
    else if ($type == 'language') {
      $value = empty($value) ? t('Language neutral') : module_invoke('locale', 'language_name', $value);
    }
    else {
      $value = $filters[$type]['options'][$value];
    }
    if ($i++) {
      $form['filters']['current'][] = array('#value' => t('<em>and</em> where <strong>%a</strong> is <strong>%b</strong>', array('%a' => $filters[$type]['title'], '%b' => $value)));
    }
    else {
      $form['filters']['current'][] = array('#value' => t('<strong>%a</strong> is <strong>%b</strong>', array('%a' => $filters[$type]['title'], '%b' => $value)));
    }
    if (in_array($type, array('type', 'language'))) {
      // Remove the option if it is already being filtered on.
      unset($filters[$type]);
    }
  }

  foreach ($filters as $key => $filter) {
    $names[$key] = $filter['title'];
    $form['filters']['status'][$key] = array('#type' => 'select', '#options' => $filter['options']);
  }

  $form['filters']['filter'] = array('#type' => 'radios', '#options' => $names, '#default_value' => 'status');
  $form['filters']['buttons']['submit'] = array('#type' => 'submit', '#value' => (count($session) ? t('Refine') : t('Filter')));
  if (count($session)) {
    $form['filters']['buttons']['undo'] = array('#type' => 'submit', '#value' => t('Undo'));
    $form['filters']['buttons']['reset'] = array('#type' => 'submit', '#value' => t('Reset'));
  }

  drupal_add_js('misc/form.js', 'core');

  return $form;
}

/**
 * Process result from node administration filter form.
 */
function icl_dashboard_filter_form_submit($form, &$form_state) {
  $_SESSION['icl_dashboard_overview_filter'] = array();
  $filters = node_filters();
  switch ($form_state['values']['op']) {
    case t('Filter'):
    case t('Refine'):
      if (isset($form_state['values']['filter'])) {
        $filter = $form_state['values']['filter'];

        // Flatten the options array to accommodate hierarchical/nested options.
        $flat_options = form_options_flatten($filters[$filter]['options']);

        if (isset($flat_options[$form_state['values'][$filter]])) {
          $_SESSION['icl_dashboard_overview_filter'][] = array($filter, $form_state['values'][$filter]);
        }
      }
      break;
    case t('Undo'):
      array_pop($_SESSION['icl_dashboard_overview_filter']);
      break;
    case t('Reset'):
      $_SESSION['icl_dashboard_overview_filter'] = array();
      break;
  }
}

/**
 * Build query for node administration filters based on session.
 */
function icl_node_build_filter_query() {
  $filters = node_filters();

  // Build query
  $where = $args = array();
  $join = '';
  foreach ($_SESSION['icl_dashboard_overview_filter'] as $index => $filter) {
    list($key, $value) = $filter;
    switch ($key) {
      case 'status':
        // Note: no exploitable hole as $key/$value have already been checked when submitted
        list($key, $value) = explode('-', $value, 2);
        $where[] = 'n.'. $key .' = %d';
        break;
      case 'category':
        $table = "tn$index";
        $where[] = "$table.tid = %d";
        $join .= "INNER JOIN {term_node} $table ON n.nid = $table.nid ";
        break;
      case 'type':
        $where[] = "n.type = '%s'";
        break;
      case 'language':
        $where[] = "n.language = '%s'";
        break;
    }
    $args[] = $value;
  }
  $where = count($where) ? 'WHERE '. implode(' AND ', $where) : '';

  return array('where' => $where, 'join' => $join, 'args' => $args);
}

/**
 * The translation dashboard menu callbank
 *
 */
function icl_content_translation_dashboard() {   
  module_load_include('inc', 'node', 'node.admin');
  
  if (isset($form_state['values']['operation']) && $form_state['values']['operation'] == 'delete') {
    return node_multiple_delete_confirm($form_state, array_filter($form_state['values']['nodes']));
  }         
   
  $form = icl_node_filter_form();
  
  unset($form['filters']);
        
  $form['#theme'] = 'node_filter_form';
  $form['admin']  = icl_content_translate_nodes();
    
  return $form;
}

function icl_content_translate_nodes() {  
    
  $languages = language_list();
  $lang = $_POST['language']?$_POST['language']:key($languages);          
  $targ = _icl_core_available_targets_for_origin($lang);
  
  if($targ){
      $form['options'] = array(
        '#type' => 'fieldset',
        '#title' => t('Translate options'),
        '#prefix' => '<div class="container-inline">',
        '#suffix' => '</div>',
      );
      foreach (module_invoke_all('icanlocalize_operations') as $operation => $array) {
          if(!$targ[$array['callback arguments'][0]]) continue;
          $form['options'][$operation] = array(
            '#type' => 'checkbox',
            '#prefix' => $array['label'],
            '#default_value' => 'translation',
          );
      }  
      $form['options']['submit'] = array(
        '#type' => 'submit',
        '#value' => t('Translate selected documents'),
        '#submit' => array('icl_content_dashboard_nodes_submit'),
      );
  }   
  
  $destination = drupal_get_destination();
    
  $filter = icl_node_build_filter_query(); 

  // get content types
  $res = db_query("SELECT type FROM {node_type}");
  $content_types = array();  
  while($row = db_fetch_object($res)){
      // see if translation is enabled
      // eliminate content types that don't have the translation enabled
      $rs = db_query("SELECT value FROM {variable} WHERE name='language_content_type_{$row->type}'");
      $rw = db_fetch_object($rs);
      $value = unserialize($rw->value);
      if($value == 2){
          $content_types[] = "'" . $row->type . "'";
      }      
  }
  
  if(empty($content_types)){
      $filter['where'] = 'WHERE 0';
  }else{
      $filter['where'] = 'WHERE n.type IN (' . join(',', $content_types) . ')';
  }
  
  if($_POST){
      if($_POST['language']){
          $lang = addslashes($_POST['language']);
      }
      
      if($_POST['translation']=='update'){
          $filter['join'] .= 'INNER JOIN {icl_content_status} cs ON n.nid = cs.nid 
                                INNER JOIN {icl_core_status} c ON c.rid = cs.rid 
                                INNER JOIN {icl_node} md ON md.nid = n.nid';
          $filter['where'] .= " AND md.md5 <> cs.md5 AND c.status = " . ICL_STATUS_SUCCESSFUL;
      }
      
      if($_POST['filter-by']){
          foreach($_POST['filter-by'] as $k=>$v){
              switch($v){
                  case 'type':
                    $filter['where'] .= " AND n.type='".$_POST['type']."'";
                    break;
                  case 'status':
                    $exp = explode('-',$_POST['status']);
                    $filter['where'] .= " AND n.".$exp[0]."='".$exp[1]."'";
              }
          }
      }
            
  }
  $filter['where'] .= " AND n.language='$lang'";
  $result = pager_query(db_rewrite_sql('SELECT n.*, u.name FROM {node} n '. $filter['join'] .' INNER JOIN {users} u ON n.uid = u.uid '. $filter['where'] .' ORDER BY n.changed DESC'), 50, 0, NULL, $filter['args']);
  $nodes = array();
  while ($node = db_fetch_object($result)) {
    $nodes[$node->nid] = '';
    $options = empty($node->language) ? array() : array('language' => $languages[$node->language]);
    $form['title'][$node->nid] = array('#value' => l($node->title, 'node/'. $node->nid, $options) .' '. theme('mark', node_mark($node->nid, $node->changed)));
    $form['name'][$node->nid] =  array('#value' => check_plain(node_get_types('name', $node)));
    $form['status'][$node->nid] =  array('#value' => ($node->status ? t('Published') : t('Not published')));
    $form['translation'][$node->nid] = array('#value' => empty($node->language) ? t('Language neutral') : theme('icl_content_fancy_status', $node, TRUE));
    
  }
  $form['nodes'] = array('#type' => 'checkboxes', '#options' => $nodes);
  $form['pager'] = array('#value' => theme('pager', NULL, 50, 0));  
  $form['#theme'] = 'icl_content_admin_nodes';
              
  return $form;
}

function icl_content_dashboard_nodes_validate($form, &$form_state) {
  $nodes = array_filter($form_state['values']['nodes']);
  if (count($nodes) == 0) {
    form_set_error('', t('No items selected.'));
  }     
}

function icl_content_dashboard_nodes_submit($form, &$form_state) {
  $form_state['rebuild'] = TRUE;
  $operations = module_invoke_all('icanlocalize_operations');  
  // Filter out unchecked nodes
  $nodes = array_filter($form_state['values']['nodes']);  
  foreach($form_state['values'] as $k=>$v){
      if(0!==strpos($k,'translation_') || !$v) continue;
      $langcodes[] = str_replace('translation_','',$k);
  }
  icl_content_translate_posts($nodes, $langcodes);
  cache_clear_all();
}

/**
 * Implementation of hook_icanlocalize_operations()
 *
 * @return array The array of operations
 */
function icl_content_icanlocalize_operations( ) {
  $langs = _icl_core_available_targets(); 
  $operations = array();
  foreach ($langs as $lang) {
    $operations['translation_'.$lang->code] = array(
	    'label' => t( 'Translate to @lang', array('@lang' => $lang->name)  ),
	    'callback' => 'icl_content_translate_posts',
	    'callback arguments' => array(
        $lang->code,
      )
	  );	  
  }  

  return $operations;
}

/* ================[ Helper functions ]=============== */

/**
 * 
 */
function icl_content_get_fancy_date($timestamp) {
  $difference = time() - $timestamp;
  
  if ($difference < 60) {
    return t('Not a minute ago');
  }
  else if ($difference < 3600) {
    return format_plural(round($difference / 60), '1 minute ago', '@count minutes ago');    
  }
  else if ($difference < 86400) {
    return format_plural(round($difference / 3600), '1 hour ago', '@count hours ago');
  }
  else {
    list($date, $time) = explode(' - ', format_date($timestamp, 'small'));
    return $date;
  }
}

/* =============[ Themable function ]============== */

function theme_icl_content_admin_nodes($form) {
  // If there are rows in this form, then $form['title'] contains a list of
  // the title form elements.
  $has_posts = isset($form['title']) && is_array($form['title']);
  $select_header = $has_posts ? theme('table_select_header_cell') : '';
  $header = array($select_header, t('Title'), t('Type'), t('Status'), t('Translation'));
  
  if ($has_posts) {
    foreach (element_children($form['title']) as $key) {
      $row = array();      
      $row[] = drupal_render($form['nodes'][$key]);
      $row[] = drupal_render($form['title'][$key]);
      $row[] = drupal_render($form['name'][$key]);
      $row[] = drupal_render($form['status'][$key]);
      $row[] = drupal_render($form['translation'][$key]);      
      $rows[] = $row;
    }
    
  }
  else {
    $rows[] = array(array('data' => t('No posts available.'), 'colspan' => '6'));
  }
  
  $fform =  icl_node_filter_form();
  
  if(!$_REQUEST['language']){
      $res = db_query("SELECT language FROM {languages} WHERE formula=''");
      $row = db_fetch_object($res);
      $sel_lang = $row->language;
  }else{
      $sel_lang = $_REQUEST['language'];
  }
  
  $sel_trans = $_REQUEST['translation']=='update'?'update':'all';
  if($_REQUEST['filter-by']){
      foreach($_REQUEST['filter-by'] as $k=>$v){
            $v1 = 'chk_' . $v;
            $v2 = 'sel_' . $v;
            $$v1 = true;          
            $$v2 = $_REQUEST[$v];
      }
  }
  
  $output .= '<fieldset>';
  $output .= '<table>';
  $output .= '<tr>';
  $output .= '<td>' . t('Show documents in: ') . '</td>';
  $output .= '<td>';
  foreach((array)$fform['filters']['status']['language']['#options'] as $k=>$v){
      if(!$k) continue;
      if($k==$sel_lang){
          $chk = 'checked="checked"';
      }else{
          $chk = '';
      }
      $output .= '<label><input type="radio" name="language" value="'.$k.'" '.$chk.'/> ' . $v . '</label> ';
  }
  $output .= '</td>';
  $output .= '</tr>';
  $output .= '<tr>';
  $output .= '<td>' . t('Translation status: ') . '</td>';
  $output .= '<td>';
  $chk = $sel_trans=='all'?'checked="checked"':'';
  $output .= '<label><input type="radio" name="translation" value="all" '.$chk.' /> ' . t('All documents') . '</label> ';
  $chk = $sel_trans=='update'?'checked="checked"':'';
  $output .= '<label><input type="radio" name="translation" value="update" '.$chk.' /> ' . t('Translation needs update') . '</label> ';
  $output .= '</td>';
  $output .= '</tr>';
  $output .= '<tr>';
  $output .= '<td valign="top">' . t('Filter further by: ') . '</td>';
  $output .= '<td>';
  $chk = $chk_status?'checked="checked"':'';
  $output .= '<label><input type="checkbox" name="filter-by[]" value="status"  '.$chk.' />' . t('Status: ') . '</label>';
  $output .= '<select name="status">';
  foreach($fform['filters']['status']['status']['#options'] as $k=>$v){
      if(0===strpos($k,'translate-')) continue;
      if($chk && $k==$sel_status){
          $sel = 'selected="selected"';
      }else{
          $sel = '';
      }
      $output .= '<option value="'.$k.'" ' . $sel . '>' . $v . '</option>';
  }
  $output .= '</select>';
  $output .= '<br />';
  $chk = $chk_type?'checked="checked"':'';
  $output .= '<label><input type="checkbox" name="filter-by[]" value="type" ' . $chk .' />' . t('Type: ') . '</label>';
  $output .= '<select name="type">';
  foreach($fform['filters']['status']['type']['#options'] as $k=>$v){
      $rs = db_query("SELECT value FROM {variable} WHERE name='language_content_type_{$k}'");
      $rw = db_fetch_object($rs);
      $value = unserialize($rw->value);
      if(2 != $value) continue;      
      if($chk && $k==$sel_type){
          $sel = 'selected="selected"';
      }else{
          $sel = '';
      }      
      $output .= '<option value="'.$k.'" '.$sel.' >' . $v . '</option>';
  }  
  $output .= '</select>';
  /*
  $output .= '<br />';
  $output .= '<label><input type="checkbox" name="filter-by[]" value="category" />' . t('Category: ') . '</label>';
  $output .= '<select name="category">';
  foreach($fform['filters']['status']['type']['#options'] as $k=>$v){
      $output .= '<option value="'.$k.'" > ' . $v . '</option> ';
  }  
  $output .= '</select>';  
  */
  $output .= '</td>';
  $output .= '</tr>';
  $output .= '<tr>';
  $output .= '<td colspan="2" align="right"><input type="submit" name="op" class="form-submit" value="'.t('Update display').'" /></td>';
  $output .= '</tr>';
  $output .= '</table>';
  $output .= '</fieldset>';
  
  
  $output .= theme('table', $header, $rows);
  if ($form['pager']['#value']) {
    $output .= drupal_render($form['pager']);
  }

  if ($has_posts) {
    $output .= drupal_render($form['options']);  
    $output .= drupal_render($form);
  }

  return $output;
}